@@ -28,6 +28,7 @@
 #include <QDebug>
 
 #define SIGNAL_CLASS_NAME "Signal"
+#define SIGNAL_INSTANCE_NAME "SignalInstance"
 #define QT_SIGNAL_SENTINEL '2'
 
 
@@ -133,7 +134,7 @@ static PyMappingMethods SignalInstance_as_mapping = {
 PyTypeObject PySideSignalInstanceType = {
     PyObject_HEAD_INIT(0)
     /*ob_size*/             0,
-    /*tp_name*/             "PySide.QtCore."SIGNAL_CLASS_NAME,
+    /*tp_name*/             "PySide.QtCore."SIGNAL_INSTANCE_NAME,
     /*tp_basicsize*/        sizeof(PySideSignalInstance),
     /*tp_itemsize*/         0,
     /*tp_dealloc*/          0,
@@ -152,7 +153,7 @@ PyTypeObject PySideSignalInstanceType = {
     /*tp_setattro*/         0,
     /*tp_as_buffer*/        0,
     /*tp_flags*/            Py_TPFLAGS_DEFAULT,
-    /*tp_doc*/              SIGNAL_CLASS_NAME,
+    /*tp_doc*/              SIGNAL_INSTANCE_NAME,
     /*tp_traverse*/         0,
     /*tp_clear*/            0,
     /*tp_richcompare*/      0,
@@ -285,7 +286,7 @@ PyObject* signalInstanceConnect(PyObject* self, PyObject* args, PyObject* kwds)
     static const char *kwlist[] = {"slot", "type", 0};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds,
-        "O|O:"SIGNAL_CLASS_NAME, (char**) kwlist, &slot, &type))
+        "O|O:"SIGNAL_INSTANCE_NAME, (char**) kwlist, &slot, &type))
         return 0;
 
     PySideSignalInstance *source = reinterpret_cast<PySideSignalInstance*>(self);
@@ -473,6 +474,13 @@ void init(PyObject* module)
     Py_INCREF(&PySideSignalInstanceType);
 }
 
+bool checkType(PyObject* pyObj)
+{
+    if (pyObj)
+        return PyType_IsSubtype(pyObj->ob_type, &PySideSignalType);
+    return false;
+}
+
 void updateSourceObject(PyObject* source)
 {
     PyTypeObject * objType = reinterpret_cast<PyTypeObject *>(PyObject_Type(source));
@@ -509,9 +517,11 @@ char* getTypeName(PyObject* type)
             else if (objType == &PyLong_Type)
                 typeName = strdup("long");
             else if (objType == &PyFloat_Type)
-                typeName = strdup("qreal");
+                typeName = strdup("double");
             else if (objType == &PyBool_Type)
                 typeName = strdup("bool");
+            else if (objType->ob_type == &SbkEnumType_Type)
+                typeName = strdup(Shiboken::Enum::getCppName(objType));
             else
                 typeName = strdup("PyObject");
         }
@@ -652,7 +662,11 @@ PySideSignal* newObject(const char* name, ...)
     sig = va_arg(listSignatures, char*);
 
     while(sig != NULL) {
-        appendSignature(self, strdup(sig));
+        if (strcmp(sig, "void") == 0)
+            appendSignature(self, strdup(""));
+        else
+            appendSignature(self, strdup(sig));
+
         sig = va_arg(listSignatures, char*);
     }
 
@@ -720,8 +734,8 @@ QStringList getArgsFromSignature(const char* signature, bool* isShortCircuit)
 
 QString getCallbackSignature(const char* signal, QObject* receiver, PyObject* callback, bool encodeName)
 {
-    QString functionName;
-    QString signature;
+    QByteArray functionName;
+    QByteArray signature;
     QStringList args;
     int numArgs = -1;
     bool useSelf = false;
@@ -731,7 +745,7 @@ QString getCallbackSignature(const char* signal, QObject* receiver, PyObject* ca
     if (isMethod || isFunction) {
         PyObject* function = isMethod ? PyMethod_GET_FUNCTION(callback) : callback;
         PyCodeObject* objCode = reinterpret_cast<PyCodeObject*>(PyFunction_GET_CODE(function));
-        functionName = PyString_AS_STRING(objCode->co_name);
+        functionName = PyString_AS_STRING(reinterpret_cast<PyFunctionObject*>(function)->func_name);
         useSelf = isMethod;
         numArgs = objCode->co_flags & CO_VARARGS ? -1 : objCode->co_argcount;
     } else if (PyCFunction_Check(callback)) {
@@ -742,25 +756,26 @@ QString getCallbackSignature(const char* signal, QObject* receiver, PyObject* ca
         if (receiver) {
             //Search for signature on metaobject
             const QMetaObject *mo = receiver->metaObject();
+            QByteArray prefix(functionName);
+            prefix += '(';
             for(int i=0; i < mo->methodCount(); i++) {
-            QMetaMethod me = mo->method(i);
-                if (QString(me.signature()).startsWith(functionName)) {
-                    numArgs = me.parameterTypes().size()+1;
+                QMetaMethod me = mo->method(i);
+                if ((strncmp(me.signature(), prefix, prefix.size()) == 0) &&
+                    QMetaObject::checkConnectArgs(signal, me.signature())) {
+                    numArgs = me.parameterTypes().size() + useSelf;
                     break;
                 }
            }
         }
 
         if (numArgs == -1) {
-            if (flags & METH_O)
-                numArgs = 1;
-            else if (flags & METH_VARARGS)
+            if (flags & METH_VARARGS)
                 numArgs = -1;
             else if (flags & METH_NOARGS)
                 numArgs = 0;
         }
     } else if (PyCallable_Check(callback)) {
-        functionName = "__callback"+QString::number((size_t)callback);
+        functionName = "__callback"+QByteArray::number((qlonglong)callback);
     }
 
     Q_ASSERT(!functionName.isEmpty());
@@ -768,7 +783,7 @@ QString getCallbackSignature(const char* signal, QObject* receiver, PyObject* ca
     bool isShortCircuit = false;
 
     if (encodeName)
-        signature = codeCallbackName(callback, functionName);
+        signature = qPrintable(codeCallbackName(callback, functionName));
     else
         signature = functionName;
 
@@ -778,7 +793,7 @@ QString getCallbackSignature(const char* signal, QObject* receiver, PyObject* ca
         signature.append('(');
         if (numArgs == -1)
             numArgs = std::numeric_limits<int>::max();
-        while (args.count() && args.count() > numArgs - useSelf) {
+        while (args.count() && (args.count() > (numArgs - useSelf))) {
             args.removeLast();
         }
         signature.append(args.join(","));
